@@ -7,6 +7,7 @@ use crate::node::DocNode;
 use crate::node::ModuleDoc;
 use crate::swc_util::get_location;
 use crate::swc_util::js_doc_for_span;
+use crate::swc_util::module_js_doc_for_source;
 use crate::ImportDef;
 use crate::Location;
 use crate::ReexportKind;
@@ -363,10 +364,16 @@ impl<'a> DocParser<'a> {
 
         let doc_node = match &export_default_decl.decl {
           DefaultDecl::Class(class_expr) => {
-            let class_def = crate::class::class_to_class_def(
-              parsed_source,
-              &class_expr.class,
-            );
+            let (class_def, decorator_js_doc) =
+              crate::class::class_to_class_def(
+                parsed_source,
+                &class_expr.class,
+              );
+            let js_doc = if js_doc.is_empty() {
+              decorator_js_doc
+            } else {
+              js_doc
+            };
             DocNode::class(name, location, js_doc, class_def)
           }
           DefaultDecl::Fn(fn_expr) => {
@@ -399,7 +406,8 @@ impl<'a> DocParser<'a> {
   ) -> Option<DocNode> {
     match decl {
       Decl::Class(class_decl) => {
-        let (name, class_def) =
+        // declared classes cannot have decorators, so we ignore that return
+        let (name, class_def, _) =
           super::class::get_doc_for_class_decl(parsed_source, class_decl);
         let js_doc = js_doc_for_span(parsed_source, &class_decl.class.span);
         let location = get_location(parsed_source, class_decl.class.span.lo);
@@ -661,6 +669,13 @@ impl<'a> DocParser<'a> {
 
     let mut is_ambient = true;
 
+    // check to see if there is a module level JSDoc for the source file
+    if let Some((js_doc, span)) = module_js_doc_for_source(parsed_source) {
+      let doc_node =
+        DocNode::module_doc(get_location(parsed_source, span.lo), js_doc);
+      doc_entries.push(doc_node);
+    }
+
     for node in module_body.iter() {
       match node {
         ModuleItem::Stmt(stmt) => {
