@@ -676,6 +676,57 @@ async fn variable_syntax() {
 mod serialization {
   use crate::*;
 
+  json_test!(module_docs,
+  r#"/* a non-jsdoc comment */
+
+/**
+ * A leading JSDoc comment
+ * @module
+ */
+
+/** One associated with a class */
+export class A {}
+    "#;
+  [
+    {
+      "kind": "moduleDoc",
+      "name": "",
+      "location": {
+        "filename": "file:///test.ts",
+        "line": 3,
+        "col": 0,
+      },
+      "jsDoc": {
+        "doc": "A leading JSDoc comment",
+        "tags": [{
+          "kind": "module"
+        }],
+      }
+    }, {
+      "kind": "class",
+      "name": "A",
+      "location": {
+        "filename": "file:///test.ts",
+        "line": 9,
+        "col": 0,
+      },
+      "jsDoc": {
+        "doc": "One associated with a class",
+      },
+      "classDef": {
+        "isAbstract": false,
+        "constructors": [],
+        "properties": [],
+        "indexSignatures": [],
+        "methods": [],
+        "extends": null,
+        "implements": [],
+        "typeParams": [],
+        "superTypeParams": []
+      }
+    }
+  ]);
+
   json_test!(declare_namespace,
     r#"
 /** Namespace JSdoc */
@@ -1164,6 +1215,219 @@ export class Foobar extends Fizz implements Buzz, Aldrin {
     }
   }]);
 
+  json_test!(export_class_decorators,
+    r#"
+@sealed
+export class A {
+  #x = "x";
+
+  @format("Hello, %s")
+  greeting: string;
+
+  @configurable(false)
+  get x() {
+    return this.#x;
+  }
+
+  @enumerable(false)
+  greet() {
+    return "hello";
+  }
+}
+"#;
+    [{
+      "kind": "class",
+      "name": "A",
+      "location": {
+        "filename": "file:///test.ts",
+        "line": 3,
+        "col": 0,
+      },
+      "classDef": {
+        "isAbstract": false,
+        "constructors": [],
+        "properties": [
+          {
+            "tsType": {
+              "repr": "string",
+              "kind": "keyword",
+              "keyword": "string",
+            },
+            "readonly": false,
+            "accessibility": null,
+            "decorators": [
+              {
+                "name": "format",
+                "args": [
+                  "\"Hello, %s\"",
+                ],
+                "location": {
+                  "filename": "file:///test.ts",
+                  "line": 6,
+                  "col": 3,
+                }
+              }
+            ],
+            "optional": false,
+            "isAbstract": false,
+            "isStatic": false,
+            "name": "greeting",
+            "location": {
+              "filename": "file:///test.ts",
+              "line": 6,
+              "col": 2,
+            }
+          }
+        ],
+        "indexSignatures": [],
+        "methods": [
+          {
+            "accessibility": null,
+            "optional": false,
+            "isAbstract": false,
+            "isStatic": false,
+            "name": "x",
+            "kind": "getter",
+            "functionDef": {
+              "params": [],
+              "returnType": null,
+              "isAsync": false,
+              "isGenerator": false,
+              "typeParams": [],
+              "decorators": [
+                {
+                  "name": "configurable",
+                  "args": [
+                    "false"
+                  ],
+                  "location": {
+                    "filename": "file:///test.ts",
+                    "line": 9,
+                    "col": 3,
+                  }
+                }
+              ]
+            },
+            "location": {
+              "filename": "file:///test.ts",
+              "line": 9,
+              "col": 2,
+            }
+          }, {
+            "accessibility": null,
+            "optional": false,
+            "isAbstract": false,
+            "isStatic": false,
+            "name": "greet",
+            "kind": "method",
+            "functionDef": {
+              "params": [],
+              "returnType": null,
+              "isAsync": false,
+              "isGenerator": false,
+              "typeParams": [],
+              "decorators": [
+                {
+                  "name": "enumerable",
+                  "args": [
+                    "false"
+                  ],
+                  "location": {
+                    "filename": "file:///test.ts",
+                    "line": 14,
+                    "col": 3,
+                  }
+                }
+              ]
+            },
+            "location": {
+              "filename": "file:///test.ts",
+              "line": 14,
+              "col": 2,
+            }
+          }
+        ],
+        "extends": null,
+        "implements": [],
+        "typeParams": [],
+        "superTypeParams": [],
+        "decorators": [
+          {
+            "name": "sealed",
+            "location": {
+              "filename": "file:///test.ts",
+              "line": 2,
+              "col": 1,
+            }
+          }
+        ]
+      }
+    }]
+  );
+
+  json_test!(decorators_jsdoc,
+  r#"
+/** jsdoc */
+@sealed
+export class A {
+  a: string = "a";
+}
+    "#;
+  [
+    {
+      "kind": "class",
+      "name": "A",
+      "location": {
+        "filename": "file:///test.ts",
+        "line": 4,
+        "col": 0,
+      },
+      "jsDoc": {
+        "doc": "jsdoc"
+      },
+      "classDef": {
+        "isAbstract": false,
+        "constructors": [],
+        "properties": [
+          {
+            "tsType": {
+              "repr": "string",
+              "kind": "keyword",
+              "keyword": "string",
+            },
+            "readonly": false,
+            "accessibility": null,
+            "optional": false,
+            "isAbstract": false,
+            "isStatic": false,
+            "name": "a",
+            "location": {
+              "filename": "file:///test.ts",
+              "line": 5,
+              "col": 2,
+            }
+          }
+        ],
+        "indexSignatures": [],
+        "methods": [],
+        "extends": null,
+        "implements": [],
+        "typeParams": [],
+        "superTypeParams": [],
+        "decorators": [
+          {
+            "name": "sealed",
+            "location": {
+              "filename": "file:///test.ts",
+              "line": 3,
+              "col": 1,
+            }
+          }
+        ]
+      }
+    }
+  ]);
+
   json_test!(export_const,
     r#"
 /** Something about fizzBuzz */
@@ -1408,8 +1672,17 @@ export const tpl2 = `Value: ${num}`;
           "repr": "foo",
           "kind":"literal",
           "literal":{
-            "kind":"string",
-            "string":"foo"
+            "kind":"template",
+            "tsTypes": [
+              {
+                "repr": "foo",
+                "kind": "literal",
+                "literal": {
+                  "kind": "string",
+                  "string": "foo"
+                }
+              }
+            ]
           }
         },
         "kind":"const"
@@ -1853,7 +2126,23 @@ export function foo(a: string, b?: number, cb: (...cbArgs: unknown[]) => void, .
     console.log("Hello world");
 }
     "#;
-  [{
+  [
+    {
+      "kind": "moduleDoc",
+      "name": "",
+      "location": {
+        "filename": "file:///test.ts",
+        "line": 1,
+        "col": 0,
+      },
+      "jsDoc": {
+        "tags": [
+          {
+            "kind": "module"
+          }
+        ]
+      },
+    }, {
       "functionDef": {
         "isAsync": false,
         "isGenerator": false,
@@ -2029,7 +2318,7 @@ export function foo([e,,f, ...g]: number[], { c, d: asdf, i = "asdf", ...rest},
             {
               "kind": "assign",
               "key": "i",
-              "value": "<UNIMPLEMENTED>"
+              "value": "[UNSUPPORTED]"
             },
             {
               "arg": {
@@ -2058,7 +2347,7 @@ export function foo([e,,f, ...g]: number[], { c, d: asdf, i = "asdf", ...rest},
               }
             }
           },
-          "right": "<UNIMPLEMENTED>",
+          "right": "[UNSUPPORTED]",
           "tsType": null
         }
       ],
@@ -2744,6 +3033,7 @@ export { foo as bar };
 export type boolLit = false;
 export type strLit = "text";
 export type tplLit = `text`;
+export type tplLitArg = `test${number}`;
 export type numLit = 5;
     "#;
   [
@@ -2798,18 +3088,66 @@ export type numLit = 5;
           "repr": "text",
           "kind": "literal",
           "literal": {
-            "kind": "string",
-            "string": "text"
+            "kind": "template",
+            "tsTypes": [
+              {
+                "repr": "text",
+                "kind": "literal",
+                "literal": {
+                  "kind": "string",
+                  "string": "text"
+                }
+              }
+            ]
           }
         },
         "typeParams": []
       }
     }, {
       "kind": "typeAlias",
-      "name": "numLit",
+      "name": "tplLitArg",
       "location": {
         "filename": "file:///test.ts",
         "line": 5,
+        "col": 0,
+      },
+      "typeAliasDef": {
+        "tsType": {
+          "repr": "test${number}",
+          "kind": "literal",
+          "literal": {
+            "kind": "template",
+            "tsTypes": [
+              {
+                "repr": "test",
+                "kind": "literal",
+                "literal": {
+                  "kind": "string",
+                  "string": "test",
+                }
+              }, {
+                "repr": "number",
+                "kind": "keyword",
+                "keyword": "number"
+              }, {
+                "repr": "",
+                "kind": "literal",
+                "literal": {
+                  "kind": "string",
+                  "string": ""
+                }
+              }
+            ]
+          }
+        },
+        "typeParams": [],
+      }
+    }, {
+      "kind": "typeAlias",
+      "name": "numLit",
+      "location": {
+        "filename": "file:///test.ts",
+        "line": 6,
         "col": 0
       },
       "typeAliasDef": {
@@ -3111,6 +3449,211 @@ export function foo(bar: any): asserts bar {}
     ]
   );
 
+  json_test!(infer_types,
+  r#"export type Flatten<T> = T extends Array<infer U> ? U : T;"#;
+  [
+    {
+      "kind": "typeAlias",
+      "name": "Flatten",
+      "location": {
+        "filename": "file:///test.ts",
+        "line": 1,
+        "col": 0
+      },
+      "typeAliasDef": {
+        "tsType": {
+          "repr": "",
+          "kind": "conditional",
+          "conditionalType": {
+            "checkType": {
+              "repr": "T",
+              "kind": "typeRef",
+              "typeRef": {
+                "typeParams": null,
+                "typeName": "T"
+              }
+            },
+            "extendsType": {
+              "repr": "Array",
+              "kind": "typeRef",
+              "typeRef": {
+                "typeParams": [
+                  {
+                    "repr": "",
+                    "kind": "infer",
+                    "infer": {
+                      "typeParam": {
+                        "name": "U"
+                      }
+                    }
+                  }
+                ],
+                "typeName": "Array"
+              }
+            },
+            "trueType": {
+              "repr": "U",
+              "kind": "typeRef",
+              "typeRef": {
+                "typeParams": null,
+                "typeName": "U"
+              }
+            },
+            "falseType": {
+              "repr": "T",
+              "kind": "typeRef",
+              "typeRef": {
+                "typeParams": null,
+                "typeName": "T",
+              }
+            }
+          }
+        },
+        "typeParams": [
+          {
+            "name": "T"
+          }
+        ]
+      }
+    }
+  ]);
+
+  json_test!(mapped_types,
+  r#"
+export type MappedTypeWithNewProperties<Type> = {
+  readonly [Properties in keyof Type as NewKeyType]?: Type[Properties];
+};
+"#;
+  [
+    {
+      "kind": "typeAlias",
+      "name": "MappedTypeWithNewProperties",
+      "location": {
+        "filename": "file:///test.ts",
+        "line": 2,
+        "col": 0,
+      },
+      "typeAliasDef": {
+        "tsType": {
+          "repr": "",
+          "kind": "mapped",
+          "mappedType": {
+            "readonly": true,
+            "typeParam": {
+              "name": "Properties",
+              "constraint": {
+                "repr": "",
+                "kind": "typeOperator",
+                "typeOperator": {
+                  "operator": "keyof",
+                  "tsType": {
+                    "repr": "Type",
+                    "kind": "typeRef",
+                    "typeRef": {
+                      "typeParams": null,
+                      "typeName": "Type"
+                    }
+                  }
+                }
+              }
+            },
+            "nameType": {
+              "repr": "NewKeyType",
+              "kind": "typeRef",
+              "typeRef": {
+                "typeParams": null,
+                "typeName": "NewKeyType"
+              }
+            },
+            "optional": true,
+            "tsType": {
+              "repr": "",
+              "kind": "indexedAccess",
+              "indexedAccess": {
+                "readonly": false,
+                "objType": {
+                  "repr": "Type",
+                  "kind": "typeRef",
+                  "typeRef": {
+                    "typeParams": null,
+                    "typeName": "Type",
+                  }
+                },
+                "indexType": {
+                  "repr": "Properties",
+                  "kind": "typeRef",
+                  "typeRef": {
+                    "typeParams": null,
+                    "typeName": "Properties"
+                  }
+                }
+              }
+            }
+          }
+        },
+        "typeParams": [
+          {
+            "name": "Type",
+          }
+        ]
+      }
+    }
+  ]);
+
+  json_test!(import_types,
+    r#"
+export function adopt<T>(p: import("./module.ts").Pet<T>) {
+  console.log(`Adopting ${p.name}...`);
+}
+"#;
+    [
+      {
+        "kind": "function",
+        "name": "adopt",
+        "location": {
+          "filename": "file:///test.ts",
+          "line": 2,
+          "col": 0,
+        },
+        "functionDef": {
+          "params": [
+            {
+              "kind": "identifier",
+              "name": "p",
+              "optional": false,
+              "tsType": {
+                "repr": "",
+                "kind": "importType",
+                "importType": {
+                  "specifier": "./module.ts",
+                  "qualifier": "Pet",
+                  "typeParams": [
+                    {
+                      "repr": "T",
+                      "kind": "typeRef",
+                      "typeRef": {
+                        "typeParams": null,
+                        "typeName": "T"
+                      }
+                    }
+                  ]
+                }
+              }
+            }
+          ],
+          "returnType": null,
+          "isAsync": false,
+          "isGenerator": false,
+          "typeParams": [
+            {
+              "name": "T"
+            }
+          ]
+        }
+      }
+    ]
+  );
+
   json_test!(indented_with_tabs,
       r#"
 /**
@@ -3383,6 +3926,32 @@ export class Class {
     "private property"
   );
 
+  contains_test!(class_decorators,
+    r#"
+@sealed
+export class A {
+  #x = "x";
+
+  @format("Hello, %s")
+  greeting: string;
+
+  @configurable(false)
+  get x() {
+    return this.#x;
+  }
+
+  @enumerable(false)
+  greet() {
+    return "hello";
+  }
+}
+    "#;
+    "@sealed",
+    "@format(\"Hello, %s\")",
+    "@configurable(false)",
+    "@enumerable(false)"
+  );
+
   contains_test!(const_declaration,
   "export const Const = 0;";
     "const Const"
@@ -3509,6 +4078,21 @@ export function f(): Generic<[string, number]> { return {}; }
     "readonly [key: string]: number"
   );
 
+  contains_test!(type_alias_infer_type,
+    "export type Flatten<T> = T extends Array<infer U> ? U : T;";
+    "T extends Array<infer U> ? U : T"
+  );
+
+  contains_test!(type_literal_mapped_type,
+    "export type T<Type> = { readonly [P in keyof Type as NewType]: Type[P]; }";
+    "readonly [P in keyof Type as NewType]: Type[P]"
+  );
+
+  contains_test!(type_import_type,
+    "export function adopt<T>(p: import(\"./module.ts\").Pet<T>) { }";
+    "import(\"./module.ts\").Pet<T>"
+  );
+
   contains_test!(interface_declaration,
   "export interface Interface {}";
     "interface Interface"
@@ -3711,7 +4295,7 @@ export const sym = Symbol("hello");
     "const b: false",
     "const bi: 100",
     "const re: RegExp",
-    "const tpl: \"foobar\"",
+    "const tpl: `foobar`",
     "const d: Date",
     "const s2: string",
     "const n2: number",
@@ -3719,6 +4303,14 @@ export const sym = Symbol("hello");
     "const sym: symbol"
   );
 
+  contains_test!(
+    ts_template_with_args,
+    r#"
+export const tpl: `test${number}` = `test1`;
+    "#;
+    "const tpl: `test${number}`"
+  );
+
   contains_test!(
     ts_user_defined_type_guards,
     r#"
